@@ -1,6 +1,6 @@
 package OpenResty;
 
-our $VERSION = '0.005003';
+our $VERSION = '0.005004';
 
 use strict;
 use warnings;
@@ -34,7 +34,7 @@ use Encode::Guess;
 #$YAML::Syck::ImplicitBinary = 1;
 
 our ($Backend, $BackendName);
-our (%AccountFiltered, %UnsafeAccounts);
+our (%AccountFiltered, %UnsafeAccounts, %UnlimitedAccounts );
 our $Cache;
 our $UUID = Data::UUID->new;
 
@@ -103,11 +103,13 @@ sub new {
     my ($class, $cgi, $call_level) = @_;
     return bless {
         _cgi => $cgi,
+        _client_ip => $cgi->remote_host(),
         _charset => 'UTF-8',
         _call_level => $call_level,
         _dumper => $Dumper,
         _importer => $Importer,
-        _http_status => 'HTTP/1.1 200 OK'
+        _http_status => 'HTTP/1.1 200 OK',
+        _unlimited => undef
     }, $class;
 }
 
@@ -128,7 +130,6 @@ sub init {
     my ($self, $rurl) = @_;
     my $class = ref $self;
     my $cgi = $self->{_cgi};
-    my $client_ip = $cgi->remote_host();
 
     if (!$Backend || !$Backend->ping) {
         warn "Re-connecting the database...\n";
@@ -148,7 +149,6 @@ sub init {
             $builtin_params{$param} = $cgi2->param($param);
         }
     }
-    $builtin_params{_client_ip} = $client_ip;
 
     $self->{_url_params} = \%url_params;
     $self->{_builtin_params} = \%builtin_params;
@@ -358,13 +358,20 @@ sub response {
         index($ENV{HTTP_ACCEPT_ENCODING} || '', 'gzip') >= 0;
     #warn "use gzip: $use_gzip\n";
     my $http_status = $self->{_http_status};
-    print "$http_status\n";
+    if ($OpenResty::Server::IsRunning) {
+        print "$http_status\r\n";
+    } else {
+        $http_status =~ s{^\s*HTTP/\d+\.\d+\s*}{};
+        #warn "http_status: $http_status\n";
+        binmode \*STDOUT;
+        print "Status: $http_status\r\n";
+    }
+    #print "$http_status\r\n";
     # warn "$http_status";
     my $type = $self->{_type} || 'text/plain';
     #warn $s;
     my $str = '';
     if (my $bin_data = $self->{_bin_data}) {
-        binmode \*STDOUT;
         local $_;
         if (my $callback = $self->{_callback}) {
             chomp($bin_data);
@@ -562,13 +569,21 @@ sub has_model {
     my ($self, $model) = @_;
     my $user = $self->current_user;
 
-   _IDENT($model) or die "Bad model name: $model\n";
+    _IDENT($model) or die "Bad model name: $model\n";
     if ($Cache->get_has_model($user, $model)) {
         #warn "has model cache HIT\n";
         return 1;
     }
-    my $sql = [:sql| select c.oid from pg_catalog.pg_class c left join pg_catalog.pg_namespace n on n.oid = c.relnamespace where c.relkind in ('r','') and n.nspname <> 'pg_catalog' and n.nspname !~ '^pg_toast' and pg_catalog.pg_table_is_visible(c.oid) and substr(c.relname,1,1) <> '_' and c.relname = $model limit 1|];
-
+    my $sql = [:sql|
+        select c.oid
+        from pg_catalog.pg_class c left join pg_catalog.pg_namespace n on n.oid = c.relnamespace
+        where c.relkind in ('r','') and
+              n.nspname = $user and
+              pg_catalog.pg_table_is_visible(c.oid) and
+              substr(c.relname,1,1) <> '_' and
+              c.relname = $model
+              limit 1
+    |];
 
     my $ret;
     eval { $ret = $self->select($sql)->[0][0]; };
@@ -633,6 +648,14 @@ sub get_role {
     $_[0]->{_role}
 }
 
+sub set_unlimited {
+    $_[0]->{_unlimted} = shift;
+}
+
+sub is_unlimited {
+    return $_[0]->{_unlimited};
+}
+
 sub url_param {
     if (@_ > 1) {
         $_[0]->{_url_params}->{$_[1]};
@@ -660,7 +683,7 @@ OpenResty - General-purpose web service platform for web applications
 
 =head1 VERSION
 
-This document describes OpenResty 0.5.3 released on November 30, 2008.
+This document describes OpenResty 0.5.4 released on May 1, 2009.
 
 =head1 DESCRIPTION
 
@@ -829,9 +852,9 @@ generated by L<Module::Install> for CPAN building system.
 
 OpenResty takes runtime performance very seriously because we have to run it on our not-so-good servers and support lots of Yahoo! China's online products with very heavy traffic.
 
-OpenResty prefers modules with XS over pure Perl ones and use cache aggressively. It's also in favor of source-filter based solutions provided by L<Filter::QuasiQuote> to reduce the length of subroutine calling chains and the number of indirections. Finally, the restyscript compiler is also written in carefully optimized Haskell code to maximize speed.
+OpenResty prefers modules with XS over pure Perl ones and uses cache aggressively. It's also in favor of source-filter based solutions provided by L<Filter::QuasiQuote> to reduce the length of subroutine calling chains and the number of indirections. Finally, the restyscript compiler is also written in carefully optimized Haskell code to maximize speed.
 
-The benchmark results for OpenResty 0.5.3's test suite on a Pentium 4 3.0GHz machine is given below:
+The benchmark results for OpenResty 0.5.3's test suite on a PentiumIV 3.0GHz machine is given below:
 
 =over
 
@@ -939,8 +962,9 @@ For a complete list of the contributors, please see L<http://svn.openfoundry.org
 
 OpenResty is licensed under the BSD License:
 
-Copyright (c) 2007-2008, Yahoo! China EEEE Works, Alibaba Inc.
-All rights reserved.
+Copyright (c) 2007-2008, Yahoo! China EEEE Works, Alibaba Inc. All rights reserved.
+
+Copyright (c) 2007-2008, Agent Zhang (agentzh). All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
